@@ -3,6 +3,7 @@
 "Project"
 from trytond.model import ModelView, ModelSQL, fields
 from trytond.model.modelstorage import OPERATORS
+from trytond.pyson import Not, Bool, Eval, Get, Equal
 import copy
 
 
@@ -13,8 +14,8 @@ class TimesheetWork(ModelSQL, ModelView):
             'Timesheet Lines',
             depends=['timesheet_available', 'active'],
             states={
-                'invisible': 'not bool(timesheet_available)',
-                'readonly': '''not active'''
+                'invisible': Not(Bool(Eval('timesheet_available'))),
+                'readonly': Not(Bool(Eval('not active'))),
             })
     sequence = fields.Integer('Sequence')
 
@@ -23,16 +24,8 @@ class TimesheetWork(ModelSQL, ModelView):
         self._order.insert(0, ('sequence', 'ASC'))
 
         self.parent = copy.copy(self.parent)
-        if not self.parent.context:
-            self.parent.context = "{'type': locals().get('type')}"
-        elif 'type' not in self.parent.context:
-            sep = ''
-            if self.parent.context[-2] != ',':
-                sep = ','
-            self.parent.context = self.parent.context[:-1] + \
-                    sep + "'type': locals().get('type')" + \
-                    self.parent.context[-1:]
-
+        self.parent.context = copy.copy(self.parent.context)
+        self.parent.context['type'] = Eval('type')
         self._reset_columns()
 
 TimesheetWork()
@@ -52,36 +45,35 @@ class Work(ModelSQL, ModelView):
             ],
             'Type', required=True, select=1,
             states={
-                'invisible': 'context.get("type", False)',
+                'invisible': Get(Eval('context', {}), 'type', False),
             })
     party = fields.Many2One('party.party', 'Party',
             states={
-                'invisible': "type!= 'project'"
+                'invisible': Not(Equal(Eval('type'), 'project')),
             }, depends=['type'])
     party_address = fields.Many2One('party.address', 'Contact Address',
-            domain=["('party', '=', party)"],
+            domain=[('party', '=', Eval('party'))],
             states={
-                'invisible': "type != 'project'"
+                'invisible': Not(Equal(Eval('type'), 'project')),
             }, depends=['party', 'type'])
     effort = fields.Float("Effort",
             states={
-                'invisible': "type != 'task'"
+                'invisible': Not(Equal(Eval('type'), 'task')),
             }, depends=['type'], help="Estimated Effort for this work")
-    total_effort = fields.Function('get_total_effort', type='float',
-            string='Total Effort',
-            help="Estimated total effort for this work and the sub-works")
+    total_effort = fields.Function(fields.Float('Total Effort',
+        help="Estimated total effort for this work and the sub-works"),
+        'get_total_effort')
     comment = fields.Text('Comment')
-    parent = fields.Function('get_parent', fnct_inv='set_parent',
-            fnct_search='search_parent', string='Parent',
-            relation='project.work', type="many2one")
+    parent = fields.Function(fields.Many2One('project.work', 'Parent'),
+            'get_parent', setter='set_parent', searcher='search_parent')
     children = fields.One2Many('project.work', 'parent', 'Children')
     state = fields.Selection([
             ('opened', 'Opened'),
             ('done', 'Done'),
             ], 'State',
             states={
-                'invisible': "type != 'task'",
-                'required': "type == 'task'",
+                'invisible': Not(Equal(Eval('type'), 'task')),
+                'required': Equal(Eval('type'), 'task'),
             }, select=1, depends=['type'])
 
     def default_type(self, cursor, user, context=None):
@@ -102,15 +94,15 @@ class Work(ModelSQL, ModelView):
         if 'company' in self._inherit_fields:
             company_field = copy.copy(
                 self._inherit_fields['company'])
-
-            company_field[2].states={
-                'invisible': 'context.get("type", False)',
-                'readonly': '''not active'''
-            }
+            company_field[2].states = copy.copy(company_field[2].states)
+            company_field[2].states.update({
+                'invisible': Bool(Get(Eval('context', {}), 'type')),
+                'readonly': Not(Bool(Eval('active'))),
+            })
 
             self._inherit_fields['company'] = company_field
 
-    def get_parent(self, cursor, user, ids, name, arg, context=None):
+    def get_parent(self, cursor, user, ids, name, context=None):
         res = dict.fromkeys(ids, None)
         project_works = self.browse(cursor, user, ids, context=context)
 
@@ -137,19 +129,19 @@ class Work(ModelSQL, ModelView):
 
         return res
 
-    def set_parent(self, cursor, user, id, name, value, arg, context=None):
+    def set_parent(self, cursor, user, ids, name, value, context=None):
         timesheet_work_obj = self.pool.get('timesheet.work')
         if value:
-            child_project_work, parent_project_work = self.browse(
-                cursor, user, [id, value], context=context)
-            child_timesheet_work_id = child_project_work.work.id
-            parent_timesheet_work_id = parent_project_work.work.id
+            project_works = self.browse( cursor, user, ids + [value],
+                    context=context)
+            child_timesheet_work_ids = [x.work.id for x in project_works[:-1]]
+            parent_timesheet_work_id = project_works[-1].work.id
         else:
-            child_project_work = self.browse( cursor, user, id, context=context)
-            child_timesheet_work_id = child_project_work.work.id
+            child_project_works = self.browse( cursor, user, ids, context=context)
+            child_timesheet_work_ids = [x.work.id for x in child_project_works]
             parent_timesheet_work_id = False
 
-        timesheet_work_obj.write(cursor, user, child_timesheet_work_id, {
+        timesheet_work_obj.write(cursor, user, child_timesheet_work_ids, {
                 'parent': parent_timesheet_work_id
                 },
                 context=context)
@@ -159,13 +151,12 @@ class Work(ModelSQL, ModelView):
 
         project_work_domain = []
         timesheet_work_domain = []
-        for field, operator, operand in domain:
-            if field.startswith('parent.'):
-                project_work_domain.append(
-                    (field.replace('parent.', ''), operator, operand))
-            elif field == 'parent':
-                timesheet_work_domain.append(
-                    (field, operator, operand))
+        if domain[0].startswith('parent.'):
+            project_work_domain.append(
+                    (domain[0].replace('parent.', ''),)
+                    + domain[1:])
+        elif domain[0] == 'parent':
+            timesheet_work_domain.append(domain)
 
         # ids timesheet_work_domain in operand are project_work ids,
         # we need to convert them to timesheet_work ids
@@ -210,7 +201,7 @@ class Work(ModelSQL, ModelView):
 
         return [('work', 'in', tw_ids)]
 
-    def get_total_effort(self, cursor, user, ids, name, arg, context=None):
+    def get_total_effort(self, cursor, user, ids, name, context=None):
 
         all_ids = self.search(cursor, user, [
                 ('parent', 'child_of', ids),
